/****************************************************************************
 * <Novell-copyright>
 * Copyright (c) 2001 Novell, Inc. All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact Novell, Inc.
 * 
 * To contact Novell about this file by physical or electronic mail, you 
 * may find current contact information at www.novell.com.
 * </Novell-copyright>
 ****************************************************************************/

/* Helpers for agent implementation */

#ifndef HULAAGENT_H
#define HULAAGENT_H

#include <xpl.h>
#include <mdb.h>
#include <logger.h>
#include <openssl/ssl.h>
#include <hulathreadpool.h>
#include <connio.h>

/* FIXME: need to tweak these */
#define HULA_QUEUE_AGENT_MAX_CLIENTS 1024
#define HULA_QUEUE_AGENT_MIN_THREADS 5
#define HULA_QUEUE_AGENT_MAX_THREADS 20
#define HULA_QUEUE_AGENT_MIN_SLEEP (5 * 60)

/** Helpers for all agents **/  

typedef void (*HulaAgentConnectionHandler)(Connection *conn);
typedef int (*HulaAgentClientHandler)(void *client, Connection *conn);
typedef void (*HulaAgentClientFree)(void *client);

typedef enum _HulaAgentStates {
    HULA_AGENT_STATE_RUNNING, 
    HULA_AGENT_STATE_UNLOADING, 
    HULA_AGENT_STATE_STOPPING, 

    HULA_AGENT_STATE_MAX_STATES
} HulaAgentState;

typedef struct {
    const char *name;
    const char *dn;
    
    HulaAgentState state;

    MDBHandle directoryHandle;
    LoggingHandle *loggingHandle;
    SSL_CTX *sslContext;

    unsigned char officialName[MAXEMAILNAMESIZE + 1];
} HulaAgent;

int HulaAgentInit(HulaAgent *agent,
                  const char *agentName,
                  const char *agentDn);
void HulaAgentHandleSignal(HulaAgent *agent,
                           int sigtype);
void HulaAgentShutdown(HulaAgent *agent);


/**  Helpers for Queue agents **/

Connection *HulaQueueConnectionInit(HulaAgent *agent, int queue);

/* Listen on a socket, calling handler for each connection */
void HulaAgentListen(HulaAgent *agent,
                     Connection *serverConn,
                     HulaThreadPool *threadPool,
                     HulaAgentConnectionHandler handler);

/* Same for queue agents, calls QDONE when the handler is complete*/
void HulaQueueAgentListen(HulaAgent *agent,
                          Connection *serverConn,
                          HulaThreadPool *threadPool,
                          HulaAgentConnectionHandler handler);

/* Listen on a socket, call handler back with an allocated client
 * struct*/
void HulaAgentListenWithClientPool(HulaAgent *agent,
                                   Connection *serverConn,
                                   HulaThreadPool *threadPool,
                                   int clientSize,
                                   int maxClients,
                                   HulaAgentClientFree clientFree,
                                   HulaAgentClientHandler handler,
                                   void **memPool);

/* Same for queue agents, calls QDONE when the handler is complete*/
void HulaQueueAgentListenWithClientPool(HulaAgent *agent,
                                        Connection *serverConn,
                                        HulaThreadPool *threadPool,
                                        int clientSize,
                                        HulaAgentClientFree clientFree,
                                        HulaAgentClientHandler handler,
                                        void **memPool);

void HulaQueueAgentGetThreadPoolParameters(HulaAgent *agent,
                                           int *minThreads,
                                           int *maxThreads,
                                           int *minSleep);

int HulaQueueAgentHandshake(Connection *conn,
                            char *buffer,
                            char *qID,
                            int *envelopeLength);

char *HulaQueueAgentReadEnvelope(Connection *conn,
                                 char *buffer,
                                 int envelopeLength);

#define HULA_ENVELOPE_NEXT(p) \
    { (p) = (p) + strlen(p) + 1;                 \
        if (*(p) == '\0') (p)++;                 \
    }

#endif
