/****************************************************************************
 * <Novell-copyright>
 * Copyright (c) 2001 Novell, Inc. All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact Novell, Inc.
 * 
 * To contact Novell about this file by physical or electronic mail, you 
 * may find current contact information at www.novell.com.
 * </Novell-copyright>
 ****************************************************************************/

#ifndef HULATHREADPOOL_H
#define HULATHREADPOOL_H

#include <xpl.h>

typedef void (*HulaThreadPoolHandler)(void *data);

typedef struct _HulaWorkItem HulaWorkItem;

typedef struct {
    const char *name;

    XplMutex lock;    
    XplSemaphore todo;

    int minimum;
    int maximum;
    int minSleep;
    int total;

    HulaWorkItem *head;
    HulaWorkItem *tail;

    BOOL shutdown;
} HulaThreadPool;

typedef struct {
    int total;
    int queueLength;
} HulaThreadPoolStatistics;

HulaThreadPool *HulaThreadPoolNew(const char *name, 
                                  int minimum,
                                  int maximum,
                                  int minSleep);

int HulaThreadPoolAddWork(HulaThreadPool *pool,
                          HulaThreadPoolHandler handler,
                          void *data);

void HulaThreadPoolGetStatistics(HulaThreadPool *pool, HulaThreadPoolStatistics *statistics);

void HulaThreadPoolShutdown(HulaThreadPool *pool);
void HulaThreadPoolFree(HulaThreadPool *pool);


#endif
