/****************************************************************************
 * <Novell-copyright>
 * Copyright (c) 2001 Novell, Inc. All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact Novell, Inc.
 * 
 * To contact Novell about this file by physical or electronic mail, you 
 * may find current contact information at www.novell.com.
 * </Novell-copyright>
 ****************************************************************************/

#ifndef _HULA_NMAP_LIBRARY_H
#define _HULA_NMAP_LIBRARY_H

#include <mdb.h>
#include <connio.h>
#include <nmap.h>

/*
    NMAP Library Return Codes
*/

#define NMAP_ERROR_COMM                             -501  /* an IP error has occurred */
#define NMAP_ERROR_INVALID_RESPONSE_CODE            -502  /* the response code is not a 4 digit number followed by a - or space */
#define NMAP_ERROR_LINE_TOO_LONG                    -503  /* the line does not fit in the network buffer */
#define NMAP_ERROR_BUFFER_TOO_SHORT                 -504  /* the line in the network buffer is bigger than the destination buffer */
#define NMAP_ERROR_CALLBACK_FUNCTION_NOT_DEFINED    -505  /* the line in the network buffer is bigger than the destination buffer */
#define NMAP_ERROR_CALLBACK_FUNCTION_FAILED         -506  /* the line in the network buffer is bigger than the destination buffer */

/*
*/
#define NMAP_REGISTRATION_CLIENT_STACK_SIZE (128 * 1024)

/*
*/
typedef enum _RegistrationStates {
    REGISTRATION_LOADING, 
    REGISTRATION_ALLOCATING, 
    REGISTRATION_CONNECTING, 
    REGISTRATION_REGISTERING, 
    REGISTRATION_COMPLETED, 
    REGISTRATION_ABORT, 
    REGISTRATION_FAILED, 

    REGISTRATION_MAX_STATES
} RegistrationStates;

typedef BOOL (* NMAPOutOfBoundsCallback)(void *param, char *beginPtr, char *endPtr);


/* Mime stuff */

typedef struct _NmapMimeInfo {
    char type[MIME_TYPE_LEN + 1];
    char subtype[MIME_SUBTYPE_LEN + 1];
    char charset[MIME_CHARSET_LEN + 1];
    unsigned char encoding[MIME_ENCODING_LEN + 1];
    unsigned char name[MIME_NAME_LEN + 1];
    int start;
    int size;
    int headersize;  /* rfc822-specific, not used */
    int lines;       /* not used */
} NmapMimeInfo;


int NMAPParseMIMELine(char *line, NmapMimeInfo *info);


/*
*/
#define NMAPAlloc(b) ConnAlloc((b))
#define NMAPFree(c) ConnFree((c))

#define NMAPClose(c, f) ConnClose((c), (f))

#define NMAPRead(c, b, l) ConnRead((c), (b), (l))
#define NMAPReadToFile(c, f, l) ConnReadToFile((c), (f), (l))
#define NMAPReadToConn(s, d, l) ConnReadToConn((c), (d), (l))

#define NMAPWrite(c, b, l) ConnWrite((c), (b), (l))
#define NMAPWriteFromFile(c, f, l) ConnWriteFromFile((c), (f), (l))

#define NMAPSend(c, b, l) ConnSend((c), (b), (l))
#define NMAPReceive(c, b, l) ConnReceive((c), (b), (l))

/*
*/
BOOL NMAPInitialize(MDBHandle directoryHandle);
void NMAPSetEncryption(SSL_CTX *context);
SSL_CTX *NMAPSSLContextAlloc(void);

int NMAPSendCommand(Connection *conn, const unsigned char *command, size_t length);
int NMAPSendCommandF(Connection *conn, const char *format, ...);

int NMAPReadResponse(Connection *conn, unsigned char *response, size_t length, BOOL check);
int NMAPReadResponseLine(Connection *conn, unsigned char *response, size_t length, BOOL check);
int NMAPReadResponseAndCount(Connection *conn, unsigned long *count);

int NMAPRunCommandF(Connection *conn, char *response, size_t length, const char *format, ...);


/* NMAPReadAnswer and NMAPReadAnswerLine are deprecated in favor of NMAPReadResponse and NMAPReadResponseLine */
int NMAPReadAnswer(Connection *conn, unsigned char *response, size_t length, BOOL checkForResult);
int NMAPReadAnswerLine(Connection *conn, unsigned char *response, size_t length, BOOL checkForResult);

Connection *NMAPConnect(unsigned char *address, struct sockaddr_in *addr);
Connection *NMAPConnectEx(unsigned char *address, struct sockaddr_in *addr, TraceDestination *destination);
BOOL NMAPEncrypt(Connection *conn, unsigned char *response, int length, BOOL force);

BOOL NMAPAuthenticate(Connection *conn, unsigned char *response, int length);

void NMAPQuit(Connection *conn);

RegistrationStates NMAPRegister(const unsigned char *dn, unsigned long queue, unsigned short port);

#endif  /* _HULA_NMAP_LIBRARY_H */
